#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import sys
import time

from blessed import Terminal

from .core import new_query
from npustat import __version__


def check_ascend_dmi():
    """ 检测命令 ascend-dmi -i 是否能够正常工作 """
    result = os.popen("ascend-dmi -i").read()
    if len(result.strip()) <= 0:
        return False
    return True


def check_npu_smi():
    """ 检测命令 npu-smi info 是否能够正常工作 """
    result = os.popen("npu-smi info").read()
    if len(result.strip()) <= 0:
        sys.stderr.write(f"命令: npu-smi info 不存在，请检查是否正确安装了toolkit，并且正确配置了环境变量\n")
        exit(1)


def print_atlas_stat(has_ascend_dmi, json=False, debug=False, *args, **kwargs):
    """
    Display the Atlas query results into standard output.
    """
    try:
        atlas_stat = new_query(has_ascend_dmi=has_ascend_dmi, *args, **kwargs)
    except Exception as e:
        sys.stderr.write("获取 Atlas 设备信息报错。请在参数中添加上 \"--debug\" 获取报错的详情信息；"
                         "并将报错信息反馈到：https://github.com/wmc1992/atlas-stat\n")
        if debug:
            try:
                import traceback
                traceback.print_exc(file=sys.stderr)
            except Exception:
                raise e
        sys.exit(1)

    if json:
        atlas_stat.print_json(sys.stdout)
    else:
        atlas_stat.print_formatted(sys.stdout, **kwargs)


def loop_atlas_stat(has_ascend_dmi, interval=1.0, *args, **kwargs):
    term = Terminal()

    with term.fullscreen():
        while 1:
            try:
                query_start = time.time()

                # Move cursor to (0, 0) but do not restore original cursor loc
                print(term.move(0, 0), end="")
                print_atlas_stat(has_ascend_dmi=has_ascend_dmi, eol_char=term.clear_eol + os.linesep, *args, **kwargs)
                print(term.clear_eos, end="")

                query_duration = time.time() - query_start
                sleep_duration = interval - query_duration
                if sleep_duration > 0:
                    time.sleep(sleep_duration)
            except KeyboardInterrupt:
                return 0


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--json", action="store_true", default=False,
                        help="将所有结果输出为JSON格式；")

    parser.add_argument("-i", "--interval", "--watch", nargs="?", type=float, default=0,
                        help="动态刷新模式；INTERVAL为刷新间隔，单位：秒；默认每2秒刷新一次；")

    parser.add_argument("--no-header", dest="no_header", action="store_true", default=False,
                        help="是否隐藏 header 信息；header 信息包含机器名称、当前时间、版本号；"
                             "默认展示 header 信息，配置该参数后 header 信息不再展示；")

    parser.add_argument("--no-title", dest="no_title", action="store_true", default=False,
                        help="是否隐藏 title 信息；title 信息为对当前设备状态值各字段的说明；"
                             "默认展示 title 信息，配置该参数后 title 信息不再展示；")

    parser.add_argument("--use-npu-smi", dest="use_npu_smi", action="store_true", default=False,
                        help="使用命令\"npu-smi info\"获取当前设备状态值；"
                             "注意该命令无法获取到加速卡的实时功率信息；")

    parser.add_argument("--show-power", dest="show_power", action="store_false", default=True,
                        help="是否展示加速卡的功率信息，默认为展示；"
                             "配置了参数 \"--use-npu-smi\" 之后该参数无效；")

    parser.add_argument("--compact", dest="compact", action="store_true", default=False,
                        help="是否采用紧凑模式展示信息，默认为不采用；"
                             "紧凑模式下会去掉空白行及其他无意义的行，适用于加速卡较多，显示器较小，屏幕显示不下的情况；")

    parser.add_argument("--debug", action="store_true", default=False,
                        help="Debug模式时允许在程序出错的情况下打印更多的调试信息；")
    parser.add_argument("-v", "--version", action="version", version=("npustat version: %s" % __version__))
    args = parser.parse_args()

    # ---------------------------------------------------------------------------------------
    # 命令 ascend-dmi 与命令 npu-smi 的区别：
    #   1) 使用命令 ascend-dmi -i --format json 返回值为json格式，并且可获取到实时的功率信息，但是需
    #      要用户正确安装了toolbox，并且命令 ascend-dmi -i 能够正常使用；
    #   2) 使用命令 npu-smi info 获取基本信息，难点在于返回值不支持json，需要自己解析，不同的设备上
    #      展示格式可能不同，解析上有比较大可能出错；同时该命令不能获取到每个加速卡的功率信息；
    # ---------------------------------------------------------------------------------------
    if not args.use_npu_smi and check_ascend_dmi():
        has_ascend_dmi = True
    else:
        has_ascend_dmi = False
        args.show_power = False  # npu-smi info 命令无法获取到加速卡的功率信息，设置为不展示
    if not has_ascend_dmi:
        check_npu_smi()

    if args.compact:
        args.no_header = True
        args.no_title = True

    if args.interval is None:  # with default value
        args.interval = 2.0  # 默认每2秒刷新一次
    if args.interval > 0:
        args.interval = max(0.1, args.interval)
        if args.json:
            sys.stderr.write("Error: \"--json\" 和 \"-i/--interval/--watch\" 不能同时使用；\n")
            sys.exit(1)

        loop_atlas_stat(**vars(args), has_ascend_dmi=has_ascend_dmi)
    else:
        del args.interval
        print_atlas_stat(**vars(args), has_ascend_dmi=has_ascend_dmi)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re

sub_space_p = re.compile(r"[ ]{2,}")  # 用于将多个连续空格替换成单个空格


class GetEntryCardListV1:
    """

               +------------------------------------------------------------------------------+
               | npu-smi 21.0.3.1                     Version: 21.0.3.1                       |
               +-------------------+-----------------+----------------------------------------+
               | NPU     Name      | Health          | Power(W)          Temp(C)              |
               | Chip    Device    | Bus-Id          | AICore(%)         Memory-Usage(MB)     |
               +===================+=================+========================================+
    line 1 ==> | 1       310       | OK              | 12.8              49                   |
    line 2 ==> | 0       0         | 0000:01:00.0    | 0                 2621 / 8192          |
               +-------------------+-----------------+----------------------------------------+
    line 1 ==> | 1       310       | OK              | 12.8              49                   |
    line 2 ==> | 1       1         | 0000:02:00.0    | 0                 2621 / 8192          |
               +-------------------+-----------------+----------------------------------------+

    """

    # card id, chip name, health, power, temp
    pattern = r""
    pattern += r"(\d{1,2})" + r" "  # card id
    pattern += r"(\d{1,5})" + r" \| "  # part of chip name, 比如 Ascend 310、Ascend 710、Ascend 910 等，这里只有数字那部分
    pattern += r"([a-zA-Z]{2,10})" + r" \| "  # Health, 有如下五种状态：OK、Warning、Alarm、Critical或UNKNOWN
    pattern += r"([\d.]{1,10})" + r" "  # Power(W)，额定功率，不是实时功率
    pattern += r"(\d{1,3})"  # Temp(C)
    line_1_p = re.compile(pattern)

    # chip id, device id, bus id, ai core, memory usage
    pattern = r""
    pattern += r"(\d{1,2})" + r" "  # chip id
    pattern += r"(\d{1,2})" + r" \| "  # device id
    pattern += r"([\d:.]{5,20})" + r" \| "  # bus id
    pattern += r"(\d{1,3})" + r" "  # ai core
    pattern += r"(\d{1,6})[ ]*/[ ]*(\d{1,6})"  # memory usage
    line_2_p = re.compile(pattern)

    # npu-smi info 命令返回值中没有 card_type 信息，首次展示前需要调用 npu-smi info -t product -i {card_id} 命令获取该信息
    # 对 card_type 信息举例："Atlas 300I Model 3000"
    card_id_to_card_type: dict = None

    @staticmethod
    def get_card_type(all_card_ids):
        if not GetEntryCardListV1.card_id_to_card_type:
            card_id_to_card_type = {}
            for card_id in all_card_ids:
                cmd_result = os.popen(f"npu-smi info -t product -i {card_id}").read()
                arr = cmd_result.split(":")
                if len(arr) == 2:
                    card_id_to_card_type[card_id] = arr[1].strip()

            GetEntryCardListV1.card_id_to_card_type = card_id_to_card_type
        return GetEntryCardListV1.card_id_to_card_type

    def get_card_entry(self, atlas_card_info):
        line_1_list, line_2_list = [], []
        for line in atlas_card_info.split("\n"):
            line = sub_space_p.sub(" ", line)
            m1 = self.line_1_p.search(line)
            m2 = self.line_2_p.search(line)

            if m1 is not None and m2 is not None:
                raise RuntimeError(f"解析 npu-smi info 结果失败，同一行匹配上两个正则，"
                                   f"line: {line}")

            if m1:
                line_1_list.append(m1.groups())
            if m2:
                line_2_list.append(m2.groups())

        if len(line_1_list) != len(line_2_list):
            raise RuntimeError(f"解析 npu-smi info 结果失败，两个正则匹配上的行数不同\n"
                               f"{atlas_card_info}")

        all_card_ids = sorted(set([card_id for (card_id, _, _, _, _) in line_1_list]))
        card_id_to_card_type = GetEntryCardListV1.get_card_type(all_card_ids)

        card_entry_list, chip_entry_list = [], []
        card_entry = dict()

        for line1, line2 in zip(line_1_list, line_2_list):
            card_id, chip_name, health, power, temp = line1
            chip_id, device_id, bus_id, ai_core, memory_used, memory_total = line2

            if len(chip_entry_list) > 0 and chip_entry_list[-1]["card_id"] != card_id:
                tmp_card_id = chip_entry_list[-1]["card_id"]
                card_entry["card_id"] = tmp_card_id
                card_entry["type"] = card_id_to_card_type.get(tmp_card_id, "??")
                card_entry["chip_entry_list"] = chip_entry_list
                card_entry_list.append(card_entry)

                card_entry = dict()
                chip_entry_list = []

            entry = dict()
            entry["card_id"] = card_id
            entry["chip_id"] = chip_id
            entry["device_id"] = device_id
            entry["health"] = health
            entry["chip_name"] = f"Ascend {chip_name}"
            entry["temperature"] = self.get_temperature(temp)
            entry["ai_core_usage"] = self.get_ai_core_usage(ai_core)
            entry["memory_used"] = f"{memory_used} MB"
            entry["memory_total"] = f"{memory_total} MB"
            entry["power"] = self.get_power(power)
            entry["bus_id"] = bus_id
            chip_entry_list.append(entry)

        if len(chip_entry_list) > 0:
            tmp_card_id = chip_entry_list[-1]["card_id"]
            card_entry["card_id"] = tmp_card_id
            card_entry["type"] = card_id_to_card_type.get(tmp_card_id, "??")
            card_entry["chip_entry_list"] = chip_entry_list
            card_entry_list.append(card_entry)

        return card_entry_list

    def get_ai_core_usage(self, ai_core_usage):
        s = str(ai_core_usage).strip()
        if isinstance(s, str) and s.isdigit():
            s = int(s)
        return s

    def get_temperature(self, temp):
        if temp.endswith("C"):
            temp = temp[:-1].strip()
        if isinstance(temp, str) and temp.isdigit():
            temp = int(temp)
        return temp

    def get_power(self, power):
        power = power.strip()
        if power.endswith("W"):
            power = power[:-1].strip()
        try:
            power = float(power)
            return f"{power:.2f} W"
        except Exception:
            return f"{power} W"


class GetCardStatusWithNpuSmi:
    # 下面这个正则中：第一个版本是npu-smi的版本，第二个版本是驱动版本
    version_p = re.compile(r"\| npu-smi ([0-9a-z.]{2,20}?) Version: [0-9a-z.]{2,20}? \|")

    version2func = {
        "21.0.3.1": GetEntryCardListV1,

        # 默认
        "default": GetEntryCardListV1,
    }

    def new_query(self):
        atlas_card_info = os.popen("npu-smi info").read()
        version = self.get_version(atlas_card_info)

        if version in self.version2func:
            my_class = self.version2func[version]()
        else:
            my_class = self.version2func["default"]()

        entry_list = my_class.get_card_entry(atlas_card_info)
        return f"npu-smi version : {version}", entry_list

    def get_version(self, atlas_card_info):
        result_list = []
        for line in atlas_card_info.split("\n"):
            line = sub_space_p.sub(" ", line)
            my_match = self.version_p.search(line)
            if my_match:
                result_list.append(my_match)

        if len(result_list) == 1:
            return result_list[0].groups()[0]
        return None



"""
The npustat module.
"""

__version__ = "0.0.3"

from .ascend_dmi import GetCardStatusWithAscendDmi
from .cli import main, print_atlas_stat, loop_atlas_stat
from .core import AtlasCardCollection, AtlasCard
from .npu_smi import GetEntryCardListV1, GetCardStatusWithNpuSmi

__all__ = (
    "__version__",
    "AtlasCardCollection", "AtlasCard",
    "GetCardStatusWithAscendDmi",
    "GetEntryCardListV1", "GetCardStatusWithNpuSmi",
    "main", "print_atlas_stat", "loop_atlas_stat",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import locale
import os
import platform
import sys
from datetime import datetime

from blessed import Terminal
from six.moves import cStringIO as StringIO

from .ascend_dmi import GetCardStatusWithAscendDmi
from .npu_smi import GetCardStatusWithNpuSmi

IS_WINDOWS = "windows" in platform.platform().lower()


class Chip:
    """ 每个Atlas加速卡中会有多个芯片，该类表示每个芯片的信息 """

    def __init__(self, entry, term, *args, **kwargs):
        if not isinstance(entry, dict):
            raise TypeError("entry should be a dict, {} given".format(type(entry)))
        self.entry = entry

        self.term = term

    def __repr__(self):
        return self.print_to(StringIO()).getvalue()

    def keys(self):
        return self.entry.keys()

    def __getitem__(self, key):
        return self.entry[key]

    @property
    def chip_id(self):
        return self.entry["chip_id"]

    @property
    def chip_name(self):
        return self.entry["chip_name"]

    @property
    def device_id(self):
        return self.entry["device_id"]

    @property
    def health(self):
        return self.entry["health"]

    @property
    def temperature(self):
        return self.entry["temperature"]

    @property
    def ai_core_usage(self):
        return self.entry["ai_core_usage"]

    @property
    def memory_used(self):
        return self.entry["memory_used"]

    @property
    def memory_total(self):
        return self.entry["memory_total"]

    def get_color(self):
        def _conditional(cond_fn, true_value, false_value, error_value=self.term.bold_black):
            try:
                return cond_fn() and true_value or false_value
            except Exception:
                return error_value

        colors = dict()
        colors["C0"] = self.term.normal
        colors["C1"] = self.term.cyan
        colors["CBold"] = self.term.bold
        colors["ChipName"] = self.term.blue
        colors["ChipTemp"] = _conditional(lambda: self.temperature < 60, self.term.red, self.term.bold_red)
        colors["ChipMemU"] = self.term.bold_yellow
        colors["ChipMemT"] = self.term.yellow
        colors["ChipHealth"] = _conditional(lambda: self.health == "OK", self.term.green, self.term.bold_red)
        colors["ChipAICore"] = _conditional(lambda: self.ai_core_usage < 50, self.term.green, self.term.bold_green)
        return colors

    def print_to(self, fp, chip_name_width=16, device_id_width=1, *args, **kwargs):
        colors = self.get_color()

        # build one-line display information
        reps = ""
        reps += "%(C1)s[{entry[chip_id]}]%(C0)s" + " "
        reps += "%(C1)s[{entry[device_id]:>{device_id_width}}]%(C0)s" + " "
        reps += "%(ChipHealth)s{entry[health]}%(C0)s" + ", "
        reps += "%(ChipName)s{entry[chip_name]:{chip_name_width}}%(C0)s" + " |"
        reps += "%(ChipTemp)s{entry[temperature]:>3}°C%(C0)s" + ", "
        reps += "%(ChipAICore)s{entry[ai_core_usage]:>3} %%%(C0)s, "
        reps += "%(C1)s%(ChipMemU)s{entry[memory_used]:>5}%(C0)s" + " / " + "%(ChipMemT)s{entry[memory_total]:>5}%(C0)s"

        def _repr(v, none_value="??"):
            return none_value if v is None else v

        reps = reps % colors
        reps = reps.format(entry={k: _repr(v) for k, v in self.entry.items()},
                           chip_name_width=chip_name_width, device_id_width=device_id_width)
        fp.write(reps)
        return fp

    def get_print_len(self, chip_name_width=16, device_id_width=1):
        """ 获取当前芯片打印出来之后的长度 """
        my_length = len(str(self.chip_id)) + len("[]") + len(" ") + \
                    max(len(str(self.device_id)), device_id_width) + len("[]") + len(" ") + \
                    len(str(self.health)) + len(", ") + \
                    max(len(str(self.chip_name)), chip_name_width) + len(" | ") + \
                    len(str(self.temperature)) + len("°C") + len(", ") + \
                    max(len(str(self.ai_core_usage)), 3) + len(" %") + len(", ") + \
                    max(len(str(self.memory_used)), 5) + len(" / ") + max(len(str(self.memory_total)), 5)
        return my_length

    def jsonify(self):
        o = self.entry.copy()

        # todo 目前还没有任何方法获取进程信息，见: https://bbs.huaweicloud.com/forum/thread-173510-1-1.html
        # if self.entry["processes"] is not None:
        #     o["processes"] = [{k: v for (k, v) in p.items() if k != "gpu_uuid"} for p in self.entry["processes"]]
        return o


class AtlasCard:
    """ Atlas加速卡 """

    def __init__(self, entry, show_power, eol_char, term, *args, **kwargs):
        if not isinstance(entry, dict):
            raise TypeError("entry should be a dict, {} given".format(type(entry)))
        self.entry = entry

        chip_list = []
        for chip_entry in entry["chip_entry_list"]:
            chip_list.append(Chip(chip_entry, term, *args, **kwargs))
        self.chip_list = chip_list

        self.show_power = show_power
        self.eol_char = eol_char
        self.term = term

    @property
    def card_id(self):
        return self.entry["card_id"]

    @property
    def type(self):
        return self.entry["type"]

    @property
    def power(self):
        return self.entry["power"]

    def get_color(self):
        colors = dict()
        colors["C0"] = self.term.normal
        colors["C1"] = self.term.cyan
        colors["CBold"] = self.term.bold
        colors["CardType"] = self.term.bold_white
        colors["CardPower"] = self.term.magenta
        return colors

    def print_to(self, fp, card_type_width=16, chip_name_width=16, device_id_width=1, *args, **kwargs):
        colors = self.get_color()

        reps = ""
        reps += "%(C1)s[{entry[card_id]}]%(C0)s" + ", "
        reps += "%(CardType)s{entry[type]:{card_type_width}}%(C0)s, "

        if self.show_power:
            reps += "%(CardPower)s{entry[power]:>3}%(C0)s"

        def _repr(v, none_value="??"):
            return none_value if v is None else v

        reps = reps % colors
        reps = reps.format(entry={k: _repr(v) for k, v in self.entry.items()}, card_type_width=card_type_width)
        fp.write(reps)
        fp.write(self.eol_char)

        # body
        for chip in self:
            chip.print_to(fp, chip_name_width=chip_name_width, device_id_width=device_id_width)
            fp.write(self.eol_char)
        return fp

    def jsonify(self):
        result = {"card_id": self.card_id, "type": self.type, }
        if self.show_power:
            result["power"] = self.power
        result["chips"] = [c.jsonify() for c in self]
        return result

    def __len__(self):
        return len(self.chip_list)

    def __iter__(self):
        return iter(self.chip_list)

    def __getitem__(self, index):
        return self.chip_list[index]


class AtlasCardCollection:
    """ 当前机器上所有atlas加速卡的信息 """

    def __init__(self, card_entry_list, version, show_power=True, no_header=True, no_title=False,
                 eol_char=os.linesep, force_color=False, compact=False, *args, **kwargs):
        self.hostname = platform.node()
        self.query_time = datetime.now()

        self.version = version
        self.show_power = show_power
        self.no_header = no_header
        self.no_title = no_title
        self.eol_char = eol_char
        self.compact = compact

        self.term = self.get_term(force_color)
        if not no_title:
            self.title_colors = self.get_title_colors()

        atlas_card_list = []
        for card_entry in card_entry_list:
            atlas_card_list.append(AtlasCard(card_entry, show_power, eol_char, self.term, *args, **kwargs))
        self.atlas_card_list = atlas_card_list

    def get_term(self, force_color=False):
        if force_color:
            TERM = os.getenv("TERM") or "xterm-256color"
            t_color = Terminal(kind=TERM, force_styling=True)

            # workaround of issue #32 (watch doesn"t recognize sgr0 characters)
            t_color._normal = u"\x1b[0;10m"
        else:
            t_color = Terminal()  # auto, depending on isatty
        return t_color

    def get_title_colors(self):
        colors = dict()
        colors["C0"] = self.term.normal
        colors["C1"] = self.term.cyan
        colors["CBold"] = self.term.bold
        colors["CardType"] = self.term.white

        colors["ChipName"] = self.term.blue
        colors["ChipTemp"] = self.term.red
        colors["ChipMemU"] = self.term.bold_yellow
        colors["ChipMemT"] = self.term.yellow
        colors["ChipHealth"] = self.term.green
        colors["ChipAICore"] = self.term.green
        colors["CardPower"] = self.term.magenta
        return colors

    def print_header(self, fp, eol_char, term, card_type_width):
        if IS_WINDOWS:
            # no localization is available; just use a reasonable default same as str(time_str) but without ms
            time_str = self.query_time.strftime("%Y-%m-%d %H:%M:%S")
        else:
            time_format = locale.nl_langinfo(locale.D_T_FMT)
            time_str = self.query_time.strftime(time_format)
        header_template = "{t.bold_white}{hostname:{width}}{t.normal}  "
        header_template += "{time_str}  "
        header_template += "{t.bold_black}{driver_version}{t.normal}"

        header_msg = header_template.format(
            hostname=self.hostname,
            width=card_type_width + 3,  # len("[?]")
            time_str=time_str,
            driver_version=self.version,
            t=term,
        )

        fp.write(header_msg.strip())
        fp.write(eol_char)

    def print_title(self, fp, eol_char, title_len):
        if not self.compact:
            fp.write("=" * title_len)
            fp.write(eol_char)

        title = "%(C1)s[加速卡ID]%(C0)s" + ", "
        title += "%(CardType)s加速卡类型%(C0)s, "
        if self.show_power:
            title += "%(CardPower)s功率%(C0)s"
        title = title % self.title_colors
        fp.write(title.strip())
        fp.write(eol_char)

        title = "%(C1)s[芯片ID]%(C0)s" + " "
        title += "%(C1)s[DeviceID]%(C0)s" + " "
        title += "%(ChipHealth)sHealth%(C0)s" + ", "
        title += "%(ChipName)s芯片名称%(C0)s" + " | "
        title += "%(ChipTemp)s温度%(C0)s" + ", "
        title += "%(ChipAICore)sAICore%(C0)s, "
        title += "%(C1)s%(ChipMemU)s内存%(C0)s"
        title = title % self.title_colors
        fp.write(title.strip())
        fp.write(eol_char)

        if not self.compact:
            fp.write("=" * title_len)
            fp.write(eol_char)
            fp.write(eol_char)

    def print_formatted(self, fp=sys.stdout, *args, **kwargs):
        # appearance settings
        card_type_width = [len(atlas_card.entry["type"]) for atlas_card in self]
        card_type_width = max([0] + card_type_width)
        chip_name_width = [len(chip.entry["chip_name"]) for atlas_card in self for chip in atlas_card]
        chip_name_width = max([0] + chip_name_width)
        device_id_width = [len(str(chip.entry["device_id"])) for atlas_card in self for chip in atlas_card]
        device_id_width = max([0] + device_id_width)

        # header
        if not self.no_header:
            self.print_header(fp=fp, eol_char=self.eol_char, term=self.term, card_type_width=card_type_width)

        # title
        if not self.no_title:
            title_len = 66
            if self.atlas_card_list:
                if self.atlas_card_list[0]:
                    title_len = self.atlas_card_list[0][0].get_print_len(chip_name_width, device_id_width)
            self.print_title(fp=fp, eol_char=self.eol_char, title_len=title_len)

        # body
        for atlas_card in self:
            atlas_card.print_to(fp, card_type_width=card_type_width, chip_name_width=chip_name_width,
                                device_id_width=device_id_width)
            if not self.compact:
                fp.write(self.eol_char)

        # todo 现在机器上只有4张卡，测试有8张加速卡时是否会一个屏幕显示不完整
        # for atlas_card in self:
        #     atlas_card.print_to(fp, card_type_width=card_type_width, chip_name_width=chip_name_width,
        #                         device_id_width=device_id_width)
        #     if not self.compact:
        #         fp.write(self.eol_char)
        fp.flush()
        return fp

    def jsonify(self):
        return {
            "hostname": self.hostname,
            "query_time": self.query_time,
            "atlas_cards": [atlas_card.jsonify() for atlas_card in self]
        }

    def print_json(self, fp=sys.stdout):
        def date_handler(obj):
            if hasattr(obj, "isoformat"):
                return obj.isoformat()
            else:
                raise TypeError(type(obj))

        o = self.jsonify()
        json.dump(o, fp, indent=4, separators=(",", ": "), default=date_handler)
        fp.write(os.linesep)
        fp.flush()

    def __len__(self):
        return len(self.atlas_card_list)

    def __iter__(self):
        return iter(self.atlas_card_list)

    def __getitem__(self, index):
        return self.atlas_card_list[index]

    def __repr__(self):
        s = "AtlasCardCollection(host=%s, [\n" % self.hostname
        s += "\n".join("  " + str(g) for g in self.atlas_card_list)
        s += "\n])"
        return s


def new_query(has_ascend_dmi, *args, **kwargs):
    """Query the information of all the Atlas Card on local machine"""

    if has_ascend_dmi:
        version, card_entry_list = GetCardStatusWithAscendDmi().new_query()
    else:
        version, card_entry_list = GetCardStatusWithNpuSmi().new_query()

    return AtlasCardCollection(card_entry_list, version=version, *args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import os


class GetCardStatusWithAscendDmi:

    def new_query(self):
        version = self.get_version()
        card_entry_list = self.get_card_entry()
        return version, card_entry_list

    def get_version(self):
        cmd = "ascend-dmi -v"  # 获取Ascend-DMI的版本
        return os.popen(cmd).read().strip()

    def devices_to_cards(self, server_type, devices):
        cards = []
        lastone = lambda : {"type": server_type} if len(cards) == 0 else cards[-1]
        for device in devices:
            card = lastone()
            card_id = card.get("card_id")
            if card_id is None:
                card["card_id"] = device["card_id"]
                chips = [device]
                card["devices"] = chips
                cards.append(card)
            elif card_id == device["card_id"]:
                chips = card["devices"]
                chips.append(device)
                card["devices"] = chips
                card["power"] = self.get_card_power(card)
            else:
                card = {"type": server_type,
                        "card_id": device["card_id"],
                        "devices": [device]}
                cards.append(card)
        return cards

    def get_card_entry(self):
        cmd = "ascend-dmi -i --format json"  # 使用Ascend-DMI做实时信息统计
        ascend_info_json = json.loads(os.popen(cmd).read())

        card_entry_list = []
        hardware_brief = ascend_info_json.get("hardware_brief")
        if hardware_brief is None:
            return card_entry_list
        cards = hardware_brief.get("cards")
        if cards is None:
            cards = []
            server = hardware_brief.get("server")
            if server is None:
                return card_entry_list
            server_type = server.get("type")
            if server_type is None:
                return card_entry_list
            devices = server.get("devices")
            if devices is None:
                return card_entry_list
            cards = self.devices_to_cards(server_type, devices)
        for card_info in cards:
            card_entry = dict()
            card_entry["card_id"] = card_info["card_id"]
            card_entry["type"] = card_info["type"]
            card_entry["power"] = self.get_power(card_info["power"])

            chip_entry_list = []
            for chip_info in card_info["devices"]:
                chip_entry = dict()
                chip_entry["chip_id"] = chip_info["chip_id"]
                chip_entry["device_id"] = chip_info["device_id"]
                chip_entry["health"] = chip_info["health"]
                chip_entry["chip_name"] = chip_info["chip_name"]
                chip_entry["temperature"] = self.get_temperature(chip_info["temperature"])
                chip_entry["ai_core_usage"] = self.get_ai_core_usage(chip_info["ai_core_information"]["ai_core_usage"])
                chip_entry["memory_used"] = chip_info["memory_information"]["used"]
                chip_entry["memory_total"] = chip_info["memory_information"]["total"]
                chip_entry_list.append(chip_entry)
            card_entry["chip_entry_list"] = chip_entry_list
            card_entry_list.append(card_entry)
        return card_entry_list

    def get_ai_core_usage(self, ai_core_usage):
        s = str(ai_core_usage).strip()
        if s.endswith("%"):
            s = s[:-1].strip()
        if isinstance(s, str) and s.isdigit():
            s = int(s)
        return s

    def get_temperature(self, temp):
        if temp.endswith("C"):
            temp = temp[:-1].strip()
        if isinstance(temp, str) and temp.isdigit():
            temp = int(temp)
        return temp

    def parse_power(self, pw):
        power = 0.0
        pw = pw.strip()
        if pw.endswith("W"):
            pw = pw[:-1].strip()
            try:
                power = float(pw)
            except Exception:
                return power
            return power

    def get_power(self, power):
        power = self.parse_power(power)
        return f"{power:.2f} W"

    def get_card_power(self, card):
        chips = card.get("devices", {})
        power = 0.0
        for chip in chips:
            power += self.parse_power(
                chip.get("power_information", {}).get(
                    "realtime_power", "0.00 W"))
        return f"{power:.2f} W"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re

from setuptools import setup

__PATH__ = os.path.abspath(os.path.dirname(__file__))


def read_readme():
    with open("README.md") as f:
        readme_string = f.read()
    src_img_path = """<img src="./docs/npustat.png" width="80%" height="80%" alt="npustat png" align=center />"""
    tar_img_path = """<img src="https://user-images.githubusercontent.com/50796589/147898619-79ada678-6709-48f2-bdb5-9366f0db2ee0.png" width="80%" height="80%" alt="npustat png" align=center />"""
    readme_string = re.sub(src_img_path, tar_img_path, readme_string)
    return readme_string


def read_version():
    # importing npustat causes an ImportError :-)
    __PATH__ = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(__PATH__, "npustat/__init__.py")) as f:
        version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                                  f.read(), re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find __version__ string")


__version__ = read_version()
print(f"version: {__version__}")

setup_requires = []

install_requires = [
    "six>=1.7",
    "blessed>=1.17.1",  # GH-126
]

setup(
    name="npustat",
    version=__version__,
    license="MIT",
    description="An utility to monitor Huawei Atlas status and usage",
    long_description=read_readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/wmc1992/npustat",
    author="wmc1992",
    author_email="m18810541081@163.com",
    keywords="atlas npu monitoring npustat",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Topic :: System :: Monitoring",
    ],
    packages=["npustat"],
    install_requires=install_requires,
    setup_requires=setup_requires,
    entry_points={
        "console_scripts": ["npustat=npustat:main"],
    },
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.6",
)
